@@ -6,7 +6,6 @@ import re
 import shlex
 import sys
 from typing import List, Tuple, Dict
-from collections import OrderedDict
 
 import configargparse
 
@@ -23,7 +22,7 @@ except ModuleNotFoundError:
     from selector import Selector
     from paths import *
 
-__version__ = '5.3.0'
+__version__ = '5.4.0'
 
 
 class Rofimoji:
@@ -37,7 +36,8 @@ class Rofimoji:
                                    '🤘', '🤙', '🤚', '🤛', '🤜', '🤝', '🤞', '🤟', '🤦', '🤰', '🤱',
                                    '🤲', '🤳', '🤴', '🤵', '🤶', '🤷', '🤸', '🤹', '🤼', '🤽', '🤾',
                                    '🥷', '🦵', '🦶', '🦸', '🦹', '🦻', '🧍', '🧎', '🧏', '🧑', '🧒',
-                                   '🧓', '🧔', '🧕', '🧖', '🧗', '🧘', '🧙', '🧚', '🧛', '🧜', '🧝'}
+                                   '🧓', '🧔', '🧕', '🧖', '🧗', '🧘', '🧙', '🧚', '🧛', '🧜', '🧝',
+                                   '🫃', '🫄', '🫅', '🫰', '🫱', '🫲', '🫳', '🫴', '🫵', '🫶'}
 
     fitzpatrick_modifiers = {
         '': 'neutral',
@@ -67,11 +67,16 @@ class Rofimoji:
         parser.add_argument(
             '--action',
             '-a',
-            dest='action',
+            dest='actions',
             action='store',
-            choices=[action.value for action in Action],
-            default=Action.TYPE.value,
-            help='How to insert the chosen characters'
+            type=Action,
+            choices=list(Action),
+            default=[Action.TYPE],
+            nargs='*',
+            metavar='ACTION',
+            help='How to insert the chosen characters. More than one action may be specified in '
+                 'a space separated list (for example: `--action type copy`). Options: ' +
+                 ', '.join(f'"{a.value}"' for a in Action)
         )
         parser.add_argument(
             '--skin-tone',
@@ -101,13 +106,6 @@ class Rofimoji:
             default='😀 ',
             help='Set rofimoj\'s  prompt'
         )
-        parser.add_argument(
-            '--rofi-args',
-            dest='rofi_args',
-            action='store',
-            default=False,
-            help=configargparse.SUPPRESS
-        )
         parser.add_argument(
             '--selector-args',
             dest='selector_args',
@@ -197,31 +195,21 @@ class Rofimoji:
             default='Alt+i',
             help='Choose the keyboard shortcut to copy the character\'s unicode codepoint to the clipboard'
         )
+        parser.add_argument(
+            '--only-official',
+            dest='use_additional',
+            action='store_false',
+            help='Use only the official Unicode descriptions'
+        )
+        parser.set_defaults(use_additional=True)
 
         parsed_args = parser.parse_args()
 
-        # Store the additional arguments for the rofi/wofi selector if they
-        # were provided.
         if parsed_args.selector_args:
             parsed_args.selector_args = shlex.split(parsed_args.selector_args)
-
-        # If the deprecated '--rofi-args' was used and '--selector-args', was
-        # not used, let's assume the user is unaware that '--selector-args'
-        # exists. We will use those arguments as our selector_args.
-        # If they provided both, we assume they know about '--selector-args'
-        # and they need to adjust their script that calls rofimoji.
-        if parsed_args.rofi_args and not parsed_args.selector_args:
-            print(
-                "🛑 The --rofi-args option is deprecated. "
-                "Please migrate to using --selector-args exclusively."
-            )
-            parsed_args.selector_args = shlex.split(parsed_args.rofi_args)
-
-        # Use an empty list if no selector or rofi args were passed.
-        if not parsed_args.selector_args:
+        else:
             parsed_args.selector_args = []
 
-        parsed_args.action = next(action for action in Action if action.value == parsed_args.action)
         parsed_args.keybindings = {
             Action.TYPE: parsed_args.keybinding_type,
             Action.COPY: parsed_args.keybinding_copy,
@@ -262,7 +250,7 @@ class Rofimoji:
             self.execute_action(characters)
         else:
             self.choose_action_from_return_code(returncode)
-            self.save_selection_to_cache( self._parse_line(chosen_character), '')
+            self.save_selection_to_cache(self._parse_line(chosen_character), '')
             self.mode_select_skin_tone('')
 
     def mode_select_skin_tone(self, processed_character: str) -> None:
@@ -288,52 +276,68 @@ class Rofimoji:
 
     def choose_action_from_return_code(self, return_code: int):
         if return_code == 20:
-            self.args.action = Action.COPY
+            self.args.actions = [Action.COPY]
         elif return_code == 21:
-            self.args.action = Action.TYPE
+            self.args.actions = [Action.TYPE]
         elif return_code == 22:
-            self.args.action = Action.CLIPBOARD
+            self.args.actions = [Action.CLIPBOARD]
         elif return_code == 23:
-            self.args.action = Action.UNICODE
+            self.args.actions = [Action.UNICODE]
         elif return_code == 24:
-            self.args.action = Action.COPY_UNICODE
+            self.args.actions = [Action.COPY_UNICODE]
 
     def _parse_line(self, line) -> str:
         return re.match(r'^(?:\u200e(?! ))?(?P<char>.[^ ]*) .*', line).group('char')
 
     def read_character_files(self) -> str:
-        all_characters = OrderedDict()
+        all_characters = {}
 
         if self.args.frecency:
             for character in self.read_frecencies().keys():
-                all_characters[character] = ''
+                all_characters[character] = []
 
-        for file_name in self.resolve_all_files():
-            characters_from_file = self.load_from_file(file_name)
+        for file in self.resolve_all_files():
+            characters_from_file = self.load_from_file(file)
             for line in characters_from_file:
                 parsed_line = line.split(' ', 1)
-                all_characters[parsed_line[0]] = all_characters.get(parsed_line[0], '') + parsed_line[1] if 1 < len(parsed_line) else ''
+                all_characters.setdefault(parsed_line[0], []).append(parsed_line[1]) if 1 < len(parsed_line) else ''
+
+        all_characters = {character: ', '.join(descriptions) for character, descriptions in all_characters.items()}
 
         return '\n'.join(f"{key} {value}" for key, value in all_characters.items() if value != '')
 
-    def resolve_all_files(self) -> List[str]:
-        file_names = self.args.files
+    def resolve_all_files(self) -> List[Path]:
+        resolved_file_names = []
+        for file_name in self.args.files:
+            resolved_file_names += self.resolve_file(file_name)
 
-        if len(file_names) == 1 and file_names[0] == 'all':
-            file_names = [file.stem for file in (Path(__file__).parent / "data").glob("*.csv")]
+        return resolved_file_names
 
-        return file_names
+    def resolve_file(self, file_name: str) -> List[Path]:
+        resolved_file_names = []
 
-    def load_from_file(self, file_name: str) -> List[str]:
         provided_file = Path(__file__).parent / "data" / f"{file_name}.csv"
+
         if Path(file_name).expanduser().is_file():
-            actual_file_name = Path(file_name).expanduser()
+            resolved_file_names.append(Path(file_name).expanduser())
         elif provided_file.is_file():
-            actual_file_name = provided_file
+            resolved_file_names.append(provided_file)
+            custom_additional_file = custom_additional_files_location / f"{file_name}.additional.csv"
+            if custom_additional_file.is_file():
+                resolved_file_names.append(custom_additional_file)
+            provided_additional_file = Path(__file__).parent / "data" / "additional" / f"{file_name}.csv"
+            if self.args.use_additional and provided_additional_file.is_file():
+                resolved_file_names.append(provided_additional_file)
+        elif file_name == 'all':
+            nested_file_names = [self.resolve_file(file.stem) for file in (Path(__file__).parent / "data").glob("*.csv")]
+            resolved_file_names += [file_name for file_names in nested_file_names for file_name in file_names]
         else:
             raise FileNotFoundError(f"Couldn't find file {file_name!r}")
 
-        return actual_file_name.read_text().strip().split('\n')
+        return resolved_file_names
+
+    def load_from_file(self, file: Path) -> List[str]:
+        return file.read_text().strip().split('\n')
 
     def load_recent_characters(self, max: int) -> List[str]:
         try:
@@ -472,26 +476,28 @@ class Rofimoji:
             file.write(characters + '\n')
 
     def execute_action(self, characters: str) -> None:
-        if self.args.action == Action.TYPE:
-            self.typer.type_characters(characters, self.active_window)
-        elif self.args.action == Action.COPY:
-            self.clipboarder.copy_characters_to_clipboard(characters)
-        elif self.args.action == Action.CLIPBOARD:
-            self.clipboarder.copy_paste_characters(characters, self.active_window, self.typer)
-        elif self.args.action == Action.UNICODE:
-            self.typer.type_characters(self.get_codepoints(characters), self.active_window)
-        elif self.args.action == Action.COPY_UNICODE:
-            self.clipboarder.copy_characters_to_clipboard(self.get_codepoints(characters))
-        elif self.args.action == Action.STDOUT:
-            print(characters)
+        for action in self.args.actions:
+            if action == Action.TYPE:
+                self.typer.type_characters(characters, self.active_window)
+            elif action == Action.COPY:
+                self.clipboarder.copy_characters_to_clipboard(characters)
+            elif action == Action.CLIPBOARD:
+                self.clipboarder.copy_paste_characters(characters, self.active_window, self.typer)
+            elif action == Action.UNICODE:
+                self.typer.type_characters(self.get_codepoints(characters), self.active_window)
+            elif action == Action.COPY_UNICODE:
+                self.clipboarder.copy_characters_to_clipboard(self.get_codepoints(characters))
+            elif action == Action.STDOUT:
+                print(characters)
 
     def save_selection_to_cache(self, characters: str, processed_characters: str) -> None:
         with cache_file_location.open('w+') as file:
-            file.write(f'{self.args.action.value}\n{characters}\n{processed_characters}')
+            actions = ",".join(action.value for action in self.args.actions)
+            file.write(f'{actions}\n{characters}\n{processed_characters}')
 
     def load_selection_from_cache(self) -> Tuple[str, str]:
         cache = cache_file_location.read_text().split('\n')
-        self.args.action = next(action for action in Action if action.value == cache[0].strip())
+        self.args.actions = [Action(a) for a in cache[0].strip().split(",")]
         return cache[1], cache[2]
 
 
